#include "oled.h"
#ifdef HARDWARE_I2C
    #include "hard_i2c.h"
#else
    #include "soft_i2c.h"
#endif
#include "font.h"

void OLED_WriteData(uint8 dat){
    I2C_start();
    I2C_send_byte(I2C_Slave_Addr);
    I2C_send_byte(0x40);
    I2C_send_byte(dat);
    I2C_stop();
}

void OLED_SetArea(uint8 startCol,uint8 startPage,uint8 endCol,uint8 endPage){
    I2C_start();
    I2C_send_byte(I2C_Slave_Addr);
    I2C_send_byte(0x00);
    I2C_send_byte(0x21);
    I2C_send_byte(startCol);
    I2C_send_byte(endCol);
    I2C_send_byte(0x22);
    I2C_send_byte(startPage);
    I2C_send_byte(endPage);
    I2C_stop();
}

void OLED_Fills(uint8 dat){
    uint16 i;
    OLED_SetArea(0,0,127,7);
    for(i = 0; i < OLED_WIDTH * OLED_HEIGHT / 8; i++){
        OLED_WriteData(dat);
    }
}

void OLED_Clear(void){
    OLED_Fills(0x00);
}

void OLED_ShowChar(uint8 sx,uint8 sy,uint8 c){
    uint8 j;
    c = c - ' ';
    OLED_SetArea(sx,sy,(sx+7)&0xFF,(sy+1)&0xFF);
    for(j = 0; j < 16; j++){
       OLED_WriteData(font_8x16[c][j]);
    }   
}

void OLED_ShowString(uint8 sx,uint8 sy,uint8 *str){
    while(*str){
        OLED_ShowChar(sx,sy,*str);
        sx += 8;
        str++;
    }
}

void OLED_ShowCN_Char(uint8 sx,uint8 sy,uint8 fontNum){
    uint8 i;
    OLED_SetArea(sx,sy,(sx+15)&0xFF,(sy+1)&0xFF);
    for(i=0;i<32;i++)
        OLED_WriteData(font_16x16[fontNum][i]);
}

void OLED_ShowNum(uint8 sx,uint8 sy,uint32 num){
    uint8 buf[12] = {0};
    uint8 i = 11;
    buf[11] = '\0';
    while(num > 0){
        buf[--i] = (num % 10) +'0';
        num /= 10;
    }
    OLED_ShowString(sx,sy,&buf[i]);
}

//void OLED_ShowCN_String(uint8 sx,uint8 sy,uint8 *strCN){
//    uint16 index;
//    while(*strCN){
//        index = *strCN;
//        OLED_ShowCN_Char(sx,sy,&font_16x16[index]);
//        sx += 16;
//        strCN += 2;
//    }

//}
void OLED_Init(void){
    
#ifdef HARDWARE_I2C
    I2C_Init();
#endif
        
    I2C_start();
    I2C_send_byte(I2C_Slave_Addr);  //ӻַ
    I2C_send_byte(0x00);            //ֽڣģʽ
    I2C_send_byte(0xAE);            //رʾ
    I2C_send_byte(0x20);            //ô洢ģʽ
    I2C_send_byte(0x00);            //0x00-ˮƽַģʽ0x01-ֱַģʽ0x02-ҳַģʽ
    I2C_send_byte(0x21);            //еַΧֽ
    I2C_send_byte(0x00);            //ʼַ
    I2C_send_byte(OLED_WIDTH - 1);  //нַ
    I2C_send_byte(0x22);            //ҳַΧֽ
    I2C_send_byte(0x00);            //ҳʼַ
    I2C_send_byte((OLED_HEIGHT/ 8) - 1);    //ҳַ
    I2C_send_byte(0x81);            //ʾԱȶ
    I2C_send_byte(0xFF);            //Աȶֵ
    I2C_send_byte(0xA6);            //ã0xA6-ԣ0xA7-
    I2C_send_byte(0xA8);            //ö·ñ
    I2C_send_byte(0x3F);            //ֵ63
    I2C_send_byte(0xC8);            //comɨ跽ƣ0xC0-COM0-COM[N-1]0xC8֮
    I2C_send_byte(0xA1);            //ӳ䣬0xA0-еַ0ӳ䵽SEG00xA1-еַ127ӳ䵽SEG0
    I2C_send_byte(0xD3);            //ôֱʾƫֵֵΧ0-63d
    I2C_send_byte(0x00);            //ƫֵΪ0x00
    I2C_send_byte(0xD5);            //ʾʱӷƵʺ;Ƶ
    I2C_send_byte(0x80);            //ʱӷƵΪA[3:0]+1ƵA[7:4]ֵ
    I2C_send_byte(0xD9);            //Ԥ
    I2C_send_byte(0xF1);            //Ԥڣλ1A[3:0]λ2A[7:4]ֵΪ0--15
    I2C_send_byte(0xDA);            //COMӲA[5],A[4]λA[5]ӳ䣬A[4]-COM
    I2C_send_byte(0x12);            //A[5] = 0,ȡCOMӳ䣬A[4] = 1COMŵڶʹ
    I2C_send_byte(0x8D);            //ɱ
    I2C_send_byte(0x14);
    I2C_send_byte(0xAF);            //ʾ
    I2C_stop();
}

void OLED_ShowImage(uint8 * image,uint8 width,uint8 height)
{
    OLED_SetArea(0,0,(width-1)&0xFF,((height/8)-1)&0xFF);
    I2C_send_bytes(0x78,0x40, image,width*height/8);

}